package zamani.metafy.core.data;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/** 
 * CameraCalibration class
 * Deals with reading camera calibration settings from a textfile and storing them as camera calibration parameters
 * 
 * @author Michael Ferguson
 * 
 */
public class CameraCalibration {
	
	// Camera Calibration Parameters
	String date;
	String camera;
	String decenteringP1;
	String decenteringP2;
	String distortionAffineB1;
	String distortionAffineB2;
	String distortionRadialK1;
	String distortionRadialK2;
	String distortionRadialK3;
	String lengthFocal;
	String pixelSizeX;
	String pixelSizeY;
	String PrincipalPointX;
	String PrincipalPointY;
	String sizeX;
	String sizeY;

    /**
     * CameraCalibration Constructor
     * Camera calibration parameters are set on constructor call
     * @param filePath String filepath to textfile containing camera calibration settings
    */
	public CameraCalibration(String filePath) throws ParseException{
		ArrayList<String> listOfLines = readLinesFromTextFile(filePath, 30);
		
		//Set calibration parameters
		setCalibrationParameters(listOfLines);
	}
	
    /**
     * Set Camera Calibration Parameters
     * @param listOfLines ArrayList<String> list of lines in camera calibration textfile
     * @throws ParseException
    */
	public void setCalibrationParameters(ArrayList<String> listOfLines) throws ParseException{
		String dateLine = listOfLines.get(2);		
		date = extractAndFormatDateLine(dateLine);
		
		String xSizeAndPixelSizeLine = listOfLines.get(14);
		String[] xSizeAndPixelSizeArray = generalExtract(xSizeAndPixelSizeLine);	
		// x Size
		sizeX = xSizeAndPixelSizeArray[1];
		// x Pixel Size
		pixelSizeX = xSizeAndPixelSizeArray[2];
		
		String ySizeAndPixelSizeLine = listOfLines.get(15);	
		String[] ySizeAndPixelSizeArray = generalExtract(ySizeAndPixelSizeLine);		
		// y Size
		sizeY = ySizeAndPixelSizeArray[1];
		// y Pixel Size
		pixelSizeY = ySizeAndPixelSizeArray[2];
				
		String focalLengthLine = listOfLines.get(19);
		lengthFocal = generalExtract(focalLengthLine)[3];
		
		String principalPointXLine = listOfLines.get(20);
		PrincipalPointX = generalExtract(principalPointXLine)[3];
		 		
		String principalPointYLine = listOfLines.get(21);
		PrincipalPointY = generalExtract(principalPointYLine)[3];
		
		String distortionRadialk1 = listOfLines.get(22);
		distortionRadialK1 = generalExtract(distortionRadialk1)[3];
		
		String distortionRadialk2 = listOfLines.get(23);
		distortionRadialK2 = generalExtract(distortionRadialk2)[3];
		
		String distortionRadialk3 = listOfLines.get(24);
		distortionRadialK3 = generalExtract(distortionRadialk3)[3];
		
		String decenteringP1Line = listOfLines.get(25);
		decenteringP1 = generalExtract(decenteringP1Line)[3];
		
		String decenteringP2Line = listOfLines.get(26);
		decenteringP2 = generalExtract(decenteringP2Line)[3];
		
		String distortionAffineB1Line = listOfLines.get(27);
		distortionAffineB1 = generalExtract(distortionAffineB1Line)[3];
		
		String distortionAffineB2Line = listOfLines.get(28);
		distortionAffineB2 = generalExtract(distortionAffineB2Line)[3];
	}
	
    /**
     * Extract and Format Dateline
     * Formats date to day/month/year
     * @param dateLine String string containing date
     * @throws ParseException
    */
    public static String extractAndFormatDateLine(String dateLine) throws ParseException{
    	char[] dateCharArray = dateLine.toCharArray();
		int beginIndex = 0;
		int endIndex = 0;
		
		// Get index of first non whitespace character
		for (int i=0; i<dateCharArray.length; i++){
			if (dateCharArray[i]!=' '){
				beginIndex = i;
				break;
			}
		}
		
		// Get index of last non whitespace character
		for (int i=dateCharArray.length-1; i>0; i--){
			if (dateCharArray[i]!=' '){
				endIndex = i;
				break;
			}
		}
		
		// Create a newly formatted string between the first and last non whitespace characters
		dateLine = dateLine.substring(beginIndex, endIndex+1);
		
		String[] fileAndCoordinates = dateLine.split("\\s");
		// Day
		String day = fileAndCoordinates[0];
		// Month
		String monthString = fileAndCoordinates[1].substring(0,fileAndCoordinates[1].length()-1);
		Date date = new SimpleDateFormat("MMM", Locale.ENGLISH).parse(monthString);
	    Calendar cal = Calendar.getInstance();
	    cal.setTime(date);
	    String month = (cal.get(Calendar.MONTH)+1)+"";
	    
	    // Pad (1-9)
	    if (month.length()==1){
	    	monthString = "0"+month;
	    }
	    
	    else{
	    	monthString = month;
	    }
	    
		// Year
		String year = fileAndCoordinates[2];
	    
	    String dateString = day+"/"+monthString+"/"+year;
	    
	    return dateString;
    }
    
    /**
     * General Extract
     * @param string String string to extract from
    */
    public static String[] generalExtract(String string){
		char[] charArray = string.toCharArray();
		
		int beginIndex = 0;
		
		// Get index of first non whitespace character
		for (int i=0; i<charArray.length; i++){
			if (charArray[i]!=' '){
				beginIndex = i;
				break;
			}
		}
		
		// Remove leading whitespaces
		string = string.substring(beginIndex, charArray.length);
		
		String[] splitString = string.split(" +");

		return splitString;
    }
    	
    /**
     * Read a specified number of lines from a text file
     * @param filePath String file path to textfile
     * @param lines int number of lines to read
    */
    public static ArrayList<String> readLinesFromTextFile(String filePath, int lines){
    	ArrayList<String> listLines = new ArrayList<String>();
    	
    	BufferedReader br = null;
    	 
		try {
 
			String sCurrentLine;
 
			br = new BufferedReader(new FileReader(filePath));
			
			int lineCount = 0;
			while ((sCurrentLine = br.readLine()) != null && lineCount!=lines) {
				lineCount++;
				listLines.add(sCurrentLine);
			}
 
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (br != null)br.close();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
 
    	
    	return listLines;
    }
	
}
