package zamani.metafy.core.data;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Dictionary;

import org.apache.tika.exception.TikaException;
import org.apache.tika.parser.AutoDetectParser;
import org.apache.tika.parser.Parser;
import org.xml.sax.SAXException;

import zamani.metafy.core.Main;
import zamani.metafy.core.gui.BackgroundWorker;
import zamani.metafy.core.utility.FileTypes;
import zamani.metafy.core.utility.Utility;

/** 
 * DirectoryVisitor class
 * Implements FileVisitor, processes all supported files in the directory recursively
 * Populate Metadata data structures
 * 
 * @author Michael Ferguson
 * 
 */
public class DirectoryVisitor implements FileVisitor<Path> {

    private Path directoryPath;
    private Parser parser;
    private BackgroundWorker.ImportWorker importworker;
	int filesProcessedCount;
	int filesInDirectory;
    
	String fileName, filePath;
	File file;
    
    /**
     * DirectoryVisitor Constructor
	 * @param directoryPath String
	 * @param importworker BackgroundWorker.ImportWorker
	 * @param filesInDirectoryCount int
    */
    public DirectoryVisitor(String directoryPath, BackgroundWorker.ImportWorker importworker, int filesInDirectoryCount) {
         this.directoryPath = FileSystems.getDefault().getPath(directoryPath);
         
		 // Metadata Parser
		 parser = new AutoDetectParser();	
		 Main.idPrefix =  Main.identifierPrefix.getText();
		 this.importworker = importworker;
		 
		 this.filesInDirectory = filesInDirectoryCount;
		 filesProcessedCount = 0;
    }
    
    /**
     * Process Files
    */
    public void processFiles() throws IOException{
    	Files.walkFileTree(directoryPath, this);
    }
    
    /**
     * Process File
     * Populate metadata data structures with automated field and extracted data
    */
    private void processFile(){
    	 Main.filesInDataset.add(file);
    	 Main.listOfFilePathsInDataset.add(filePath);
		
		// Tempory metadata array
		String[] t = new String[ Main.basicMetadataSchema.size()];
		
		// Set Identifier
		t[ Main.basicMetadataSchema.indexOf("Identifier")] =  Main.idPrefix+":"+Utility.padString( Main.fileIndex+"",7,"0");
		
		// Set Creators
		int creatorMetadataStartIndex =  Main.basicMetadataSchema.indexOf("Creator");
		for (int i=0; i< Main.items.get( Main.indexOfCreatorInFieldComboBox).length;i++){
			t[creatorMetadataStartIndex+i]= Main.items.get( Main.indexOfCreatorInFieldComboBox)[i];
		}
		// Set Format
		t[ Main.basicMetadataSchema.indexOf("Format")] = FileTypes.getFormat(Utility.getExtension(fileName));
		// Set Source
		t[ Main.basicMetadataSchema.indexOf("Source")] =  Main.sourceOfData.getText();
		// Set Date		        
    	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        t[ Main.basicMetadataSchema.indexOf("Date")] = sdf.format(file.lastModified());
		//Set Coverage Spatial Country
        t[ Main.basicMetadataSchema.indexOf("Coverage Spatial Country")] =  Main.coverageSpatialCountry.getText();
        //Set Coverage Spatial Name
        t[ Main.basicMetadataSchema.indexOf("Coverage Spatial Name")] =  Main.coverageSpatialSitename.getText();

        //Set Type
        //--------
		// 3D Model Type
		if ((Utility.arrayContains(Utility.getExtension(fileName),FileTypes.modelType))){	
			t[ Main.basicMetadataSchema.indexOf("Type")] = "3D Models";
		}

		// Necessary to create new Arrays/References for each file (otherwise changing fields for one file will alter all files fields)
		ArrayList<String> temp = new ArrayList<String>(Arrays.asList(t));
		ArrayList<String> basicMetadataSchema = new ArrayList<String>();
		for (String schemaField : Main.basicMetadataSchema){
			basicMetadataSchema.add(schemaField);
		}
		
		 Main.metadataFields.add(basicMetadataSchema);
		 Main.metadata.add(temp);
		String[] tempArray = new String[5];
		tempArray[0] =  Main.idPrefix+":"+Utility.padString( Main.fileIndex+"",7,"0");
		tempArray[1] = Utility.getExtension(file.getName());
		tempArray[2] =  Main.index+"";
		tempArray[3] = "Status";
		tempArray[4] = "undefined";	

		// Only get Camera Model if image is an accepted Calibration Image Type
		if((Utility.arrayContains(Utility.getExtension(fileName),FileTypes.cameraCalibrationImageType))){
			 Dictionary<String, String> extractedMetadataDictionary = null;
			try {
				extractedMetadataDictionary =  DataDealer.extractFromFile(parser, file);
			} catch (IOException | SAXException | TikaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			 tempArray[4] = extractedMetadataDictionary.get("Camera Model");
		}

		 Main.listOfFileCorrespondingPIDSInDataset.add( Main.idPrefix+":"+Utility.padString( Main.fileIndex+"",7,"0"));
		
		 Main.idAndFileTypeList.add(tempArray);
		 Main.index++;
		 Main.fileIndex++;
		filesProcessedCount++;
		 Main.progressCount = ((double)filesProcessedCount/(double)filesInDirectory)*100;
		importworker.setProgressAmount((int) Main.progressCount);
    }

    @Override
    public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult visitFile(Path path, BasicFileAttributes attrs) throws IOException {
		if ((Utility.arrayContains(Utility.getExtension(path.getFileName().toString()),FileTypes.supportedFileTypes))){
			fileName = path.getFileName().toString();
			file = path.toFile();
			filePath = file.getPath();
			processFile();
		}
        return FileVisitResult.CONTINUE;
    }

    @Override
    public FileVisitResult visitFileFailed(Path file, IOException exc) throws IOException {
        return FileVisitResult.CONTINUE;
    }
}	