package zamani.metafy.core.gui;
import java.awt.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import javax.swing.*;
 
/** 
 * ImageScalingPanel class
 * Extends JPanel, Deals with Image Scaling functionality as well as Image Display
 * 
 * @author Michael Ferguson
 * 
 */
public class ImageScalingPanel extends JPanel
{
	private static final long serialVersionUID = -5273136842728502016L;
	
	BufferedImage image;
    double scale, zoomInc;
    boolean best_fit = false;
    Dimension bestFitDimension;
 
    /**
     * ImageScalingPanel Constructor
    */
    public ImageScalingPanel()
    {
        scale = 1.0;
        zoomInc = 0.01;
        setBackground(Color.white);
    }
 
    /**
     * ImageScalingPanel Paint Component
     * @param g Graphics
    */
    protected void paintComponent(Graphics g)
    {
    	if (image!=null){
	        super.paintComponent(g);
	        Graphics2D g2 = (Graphics2D)g;
	        g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION,RenderingHints.VALUE_INTERPOLATION_BICUBIC);
	        int w = getWidth();
	        int h = getHeight();
	        
	        double imageWidth = scale * image.getWidth();
	        double imageHeight = scale * image.getHeight();
	        
	        double x = (w - imageWidth)/2;
	        double y = (h - imageHeight)/2;
	        
	        AffineTransform at = AffineTransform.getTranslateInstance(x, y);
	        at.scale(scale, scale);
	        g2.drawRenderedImage(image, at);     
    	}
    	else{
    		super.paintComponent(g);
    		setBackground(Color.white);
    	}
    }
    
    /**
     * Calculate the Scale that 'Best Fits' the Image Scaling Panel
    */
    public void calculate_best_fit_scale(){
        double imageWidth = scale * image.getWidth();
        double imageHeight = scale * image.getHeight();
        
        int index = 0;
        
        while(scale * image.getWidth()>=bestFitDimension.getWidth() || scale * image.getHeight()>=bestFitDimension.getHeight()){
        	//Try with bestFit width first
        	if (index==0){
        		scale = bestFitDimension.getWidth()/imageWidth;
        	}
        	// Try with bestFit height second
        	else if (index==1){
        		scale = bestFitDimension.getHeight()/imageHeight;
        	}
        	// Set scale to smallest scale of the two
        	else if (index==2){
        		scale = Math.min(bestFitDimension.getWidth()/imageWidth, bestFitDimension.getHeight()/imageHeight);
        	}
        	else{
        		scale-=0.01;
        	}     

        	index++;
        }
    }
 
    /**
     * Set the Scale of the Image Scaling Panel
     * @param direction int scale direction
    */
    public void setScale(int direction)
    {
        scale += direction * zoomInc;
        
        if (scale>1.0){
        	scale=1.0;					// Max scale
        }
        
        if (scale<=0.0){
        	scale=0.05;					// Min scale
        }
        
        revalidate();
        repaint();
    }
    
    /**
     * Get the Preffered Size of the Image Scaling Panel
     * @return  preferred size of the Image Scaling Panel
    */
    public Dimension getPreferredSize()
    {
    	if (image!=null){
	        int
	            w = (int)(scale * image.getWidth()),
	            h = (int)(scale * image.getHeight());
	        return new Dimension(w, h);
    	}
    	else{
    		return new Dimension(100,100);
    	}
    }
    
    /**
     * Get the size of the Image Scaling Panel
     * @return  size of the Image Scaling Panel
    */
    public Dimension getSizeOfPanel(){
    	return getSize();
    }
    
    /**
     * Set the Image Scaling Panel to its normal size
    */
    public void setNormalSize(){
    	scale = 1.0;
        revalidate();
        repaint();
    }
    
    /**
     * Set the Image Scaling Panel to its best fit
     * @param bestFit boolean
     * @param d Dimension
    */
    public void setBestFit(boolean bestFit, Dimension d){
    	bestFitDimension = d;
    	best_fit = bestFit;
    	calculate_best_fit_scale();
        revalidate();
        repaint();
    }
    
    /**
     * Set the Image Scaling Panel's image
     * @param tempImage BufferedImage
    */
    public void setImage(BufferedImage tempImage)
    {
    	image = tempImage;
        revalidate();
        repaint();
    }
    
    /**
     * Reset the Image Scaling Panel
    */
    public void resetPanel(){
    	image = null;
    	setBackground(Color.white);
    	scale = 1.0;
        revalidate();
        repaint();
    }
    
    /**
     * Check if Image Scaling Panel is empty
     * @return true if panel is empty, false if not
    */
    public boolean isPanelEmpty(){
    	if (image==null){
    		return true;
    	}
    	else{
    		return false;
    	}
    }
}
